% In this example we apply the cMOSM-EB method to denoise a noisy Lena image
% of size 512x512 and noise standard deviation sigma=10
%
%
%  The program calls to:
%   MakeCONFilter.m     -- Generates coefficients of Daubechies maxflat 
%                           symmetric complex orthonormal wavelet 
%                           filters (Author: B.G. Sherlock, 1998)
%   rec_cMOSM_EB.m      -- Performs 2D complex maximum overlap 
%                           scale-mixing wavelet transform and empirical 
%                           Bayes estimation of the wavelet coefficients
%--------------------------------------------------------------------------

close all;
clear all;

L = imread('lena.png'); % Noiseless image
L = double(L); 

n = length(L);
coarsest = 3; %coarsest level of wavelet decomposition

sigma = 10; % noise standard deviation
N = sigma*normrnd(0,1,n,n); % add white noise
Ln = L + N;

% Calculate SNR (signal-to-noise ratio)
SNR = 10*log10( sum(sum(L.^2)) / sum(sum(N.^2)) );
clearvars N;

% Calculate PNSR (peak SNR) of noisy image
mse0 = sum(sum((L-Ln).^2))/(n^2);
PSNR0 = 10*log10(255^2/mse0);
disp(['SNR of the image is ' num2str(SNR) 'dB']);
disp(['PSNR of noisy image is ' num2str(PSNR0) 'dB']);

wf = MakeCONFilter(6); % complex wavelet filter

% Function performing 2D scale-mixing complex wavelet 
% transform and empirical Bayes estimation
L_tr = rec_cMOSM_EB(Ln,sigma,wf,coarsest);

% MSE and PSNR of denoised image
mse = sum(sum((L-L_tr).^2))/(n^2);
PSNR = 10*log10(255^2/mse);
disp(['PSNR of denoised image is ' num2str(PSNR) 'dB']);
disp(['MSE of denoised image is ' num2str(mse)]);

% Plotting the noisy and denoised images
figure(1);
image(Ln);
colormap(gray(256));
axis square;
title('Noisy image');
figure(2);
image(L_tr);
colormap(gray(256));
axis square;
title('Denoised image');






