#include "mex.h"
#include "math.h"

/* Modification of modtwj.c algorithm accepting complex wavelet filter by Norbert Remenyi (2012)
/* Based on the modtwj.c algorithm by Charles R. Cornish
 *
/* To compile : mex cmodwtj.c
 */

void modwtj(double *VinR, double *VinI, int N, int j, int L, double *htR, double *htI,
        double *gtR, double *gtI, double *WoutR, double *WoutI, double *VoutR, double *VoutI)
{

  int k, n, t, i;
  double *WoutRR, *WoutRI, *WoutIR, *WoutII;
  double *VoutRR, *VoutRI, *VoutIR, *VoutII;
 
  WoutRR = (double*)malloc(N*sizeof(double));
  WoutRI = (double*)malloc(N*sizeof(double));
  WoutIR = (double*)malloc(N*sizeof(double));
  WoutII = (double*)malloc(N*sizeof(double));
  VoutRR = (double*)malloc(N*sizeof(double));
  VoutRI = (double*)malloc(N*sizeof(double));
  VoutIR = (double*)malloc(N*sizeof(double));
  VoutII = (double*)malloc(N*sizeof(double));
   
  for (t = 0; t < N; t++) {
    k = t;
    WoutRR[t] = htR[0] * VinR[k];
    VoutRR[t] = gtR[0] * VinR[k];
    WoutRI[t] = htI[0] * VinR[k];
    VoutRI[t] = gtI[0] * VinR[k];
    WoutIR[t] = htR[0] * VinI[k];
    VoutIR[t] = gtR[0] * VinI[k];
    WoutII[t] = htI[0] * VinI[k];
    VoutII[t] = gtI[0] * VinI[k];
    for (n = 1; n < L; n++) {
      k -= (int) pow(2.0, (double) j - 1.0);
      if (k < 0) {
        k += N;
      }
      WoutRR[t] += htR[n] * VinR[k];
      VoutRR[t] += gtR[n] * VinR[k];
      WoutRI[t] += htI[n] * VinR[k];
      VoutRI[t] += gtI[n] * VinR[k];
      WoutIR[t] += htR[n] * VinI[k];
      VoutIR[t] += gtR[n] * VinI[k];
      WoutII[t] += htI[n] * VinI[k];
      VoutII[t] += gtI[n] * VinI[k];
    }
  }
  for(i = 0; i < N; i++) {
      VoutR[i] = VoutRR[i] - VoutII[i];
      VoutI[i] = VoutIR[i] + VoutRI[i];
      WoutR[i] = WoutRR[i] - WoutII[i];
      WoutI[i] = WoutIR[i] + WoutRI[i];
  }
  
  free(WoutRR); 
  free(WoutRI);
  free(WoutIR);
  free(WoutII);
  free(VoutRR);
  free(VoutRI);
  free(VoutIR);
  free(VoutII);
  
}

void mexFunction(int nlhs, mxArray *plhs[], 
		 int nrhs, const mxArray *prhs[])
{
  int N, j, L;
  int m, n;
  double *WoutR, *WoutI, *VoutR, *VoutI;
  double *VinR, *VinI, *htR, *htI, *gtR, *gtI;
  
  /* Check for proper number of arguments */
  
  if (nrhs != 7) {
    mexErrMsgTxt("modwtj requires seven input arguments.");
  } else if (nlhs > 4) {
    mexErrMsgTxt("modwtj requires four output arguments.");
  }

  /* Read input arguments */
  
  VinR = mxGetPr(prhs[0]);
  VinI = mxGetPr(prhs[1]);
  htR = mxGetPr(prhs[2]);
  htI = mxGetPr(prhs[3]);
  gtR = mxGetPr(prhs[4]);
  gtI = mxGetPr(prhs[5]);

  // #define DEBUG
  
  N = mxGetNumberOfElements(prhs[0]);
  #ifdef DEBUG
    mexPrintf("N = %d\n", N);
  #endif
  L = mxGetNumberOfElements(prhs[2]);

  j = (int) mxGetScalar(prhs[6]);
  #ifdef DEBUG
    mexPrintf("j = %d\n", j);
  #endif

  m = mxGetM(prhs[0]);
  #ifdef DEBUG
    mexPrintf("m = %d\n", m);
  #endif

  n = mxGetN(prhs[0]);
  #ifdef DEBUG
    mexPrintf("n = %d\n", n);
  #endif
  
  /* Create arrays for the output arguments */
  
  plhs[0] = mxCreateDoubleMatrix(m, n, mxREAL);
  plhs[1] = mxCreateDoubleMatrix(m, n, mxREAL);
  plhs[2] = mxCreateDoubleMatrix(m, n, mxREAL);
  plhs[3] = mxCreateDoubleMatrix(m, n, mxREAL);
  
  /* Assign pointers to output arrays. */
  
  WoutR = mxGetPr(plhs[0]);
  WoutI = mxGetPr(plhs[1]);
  VoutR = mxGetPr(plhs[2]);
  VoutI = mxGetPr(plhs[3]);
  
  /* Compute the modwt at the jth level. */
  
  modwtj(VinR, VinI, N, j, L, htR, htI, gtR, gtI, WoutR, WoutI, VoutR, VoutI);

  return;
}
