% This function evaluates the log-likelihood function for wavelet
% coefficients in a specific subband and for certain input values 
% of the hyperparameters; it is used to find estimates of the 
% hyperparameters by empirical Bayes method.
%
% Input
%  par  -- value of hyperparameters p_j (mixing weight) and V_j 
%           (covariance matrix) for subband j
%  dd   -- observed wavelet coefficients
%  S    -- Noise covariance matrix Sigma_j
%
% Output
%  out -- value of the negative of the log-likelihood
% -------------------------------------------------------------------------

function [out]=loglik_2D(par,dd,S)

Si = inv(S);
detS = det(S);

% Reparameterize covariance matrix off-diagonal elements to (-1,1)
C = [par(2),par(3)*sqrt(par(2)*par(4));par(3)*sqrt(par(2)*par(4)),par(4)];
Vinv = inv(C+S);
detV = det(C+S);

% Considerably speeds up evaluation of log likelihood for optimization
a = par(1)/(2*pi*sqrt( detV ));
b = 0.5*sum(dd*Vinv.*dd,2);
c = (1-par(1))/(2*pi*sqrt( detS ));
d = 0.5*sum(dd*Si.*dd,2);

% To avoid numerical problems we rewrite log( a*exp(-b) + c*exp(-d) )
out = -sum(log(a) - b + log( 1 + c./a .* exp(-d+b) ) );

